import React from "react";
import { StyleSheet, FlatList, ScrollView, View, ViewPropTypes, VirtualizedList} from "react-native";
import PropTypes from "prop-types";

const style = StyleSheet.create({
	container:{
		flex:1
	}
})

class Item extends React.PureComponent {
	render() {
		return <View style={{ flex: 1 }} onLayout={(event)=>{
			const { height } = event.nativeEvent.layout;
			this.props.addHeight?.(height, this.props.index, this.props.item.onLayout)
			}}>
			{this.props.renderItem(this.props.item)}
		</View>
	}
}

class Column extends React.PureComponent {
	static propTypes = {
		keyExtractor: PropTypes.func,
		index:PropTypes.number
	};

	constructor(props) {
		super(props);
		this.state = {
			data: []
		};
		this.height = 0
		this.addedHeightIndexArray = [];
		this.addHeight = this.addHeight.bind(this)
		this.getData = this.getData.bind(this)
	}

	clear() {
		this.addedHeightIndexArray=[];
		this.setState({ data: [], height: 0 });
	}

	render() {
		return <View style={{ flex: 1, overflow: "hidden" }}>
			<FlatList
				scrollEnabled={false}
				listKey={`flatlist_listkey_${this.props.index}`}
				style={style.container}
				data={this.state.data}
				keyExtractor={this.props.keyExtractor}
				renderItem={this.renderItem.bind(this)}
			/>
		</View>
	}

	getHeight() {
		return this.height;
	}

	getData(){
		return this.state.data
	}

	addItems(items) {
		this.setState({ data: [...this.state.data, ...items] });
	}

	addHeight( height, index, callback ) {
		newHeight = this.height + height;
		if(!this.addedHeightIndexArray.includes(index)){
			this.addedHeightIndexArray.push(index)
			this.height += newHeight;
			callback&&callback()
			// this.setState( { height: newHeight }, ()=>{callback&&callback()} );
		}
	}

	renderItem({ item, index }) {
		return <Item renderItem={this.props.renderItem} index={index} item={item} addHeight={this.addHeight}/>
	}
}

export default class Masonry extends React.PureComponent {

	static propTypes = {
		columns: PropTypes.number,
		containerStyle: ViewPropTypes.style,
		style: ViewPropTypes.style,
		renderItem: PropTypes.func,
		keyExtractor: PropTypes.func,
		onMomentumScrollEnd: PropTypes.func,
		virtualizedListProps: PropTypes.object
	};

	static defaultProps = {
		columns: 2
	};

	constructor(props) {
		super(props);
		const columns = [];
		for (let i = 0; i < props.columns; i++) {
			columns.push(null);
		}
		this.state = {
			columns
		};
		this.itemQueue = [];
		this.refArray=[];
		this.virtualizedListRenderItem = this.virtualizedListRenderItem.bind(this)
	}

	clear() {
		this.refArray.forEach((colRef)=>colRef.clear?.())
		// this.state.columns.forEach(col => col.clear());
	}

	addItems(items) {
		if (items) {
			if (this.itemQueue.length > 0) {
				this.itemQueue = this.itemQueue.concat(items);
			} else {
				this.itemQueue = this.itemQueue.concat(items);
				this.addItems();
			}
		} else {
			if (this.itemQueue.length > 0) {
				const item = this.itemQueue.shift();
				this.addItem(item, () => this.addItems());
			}
		}
	}
	
	addItemsWithHeight(items) {	
		const columns = this.refArray.map((col)=>{
			return{
				columnRef:col,
				height:col.getHeight(),
				data:[]
			}
		})

		items.forEach((item)=>{
			const col = columns.sort((a, b) => a.height - b.height)[0];
			col.data.push(item);
			col.height += item.height;
		})

		columns.forEach(col => {
			col.column.addItems(col.data);
		})
	}

	sortColumns() {
		let cloneArr = [...this.refArray].sort((a, b) => a.getHeight() - b.getHeight());
		return cloneArr[0]
	}

	addItem(item, callback) {
		const minCol = this.sortColumns()
		item.onLayout = (event) => {
			callback()
		  }
		minCol.addItems([item]);
	}

	virtualizedListGetItem(data, index) {
		return data[0]
	}

	virtualizedListGetItemCount(data) {
		return data.length
	}

	virtualizedListRenderItem(){
		return (
			<View>
				{this.props.header}
				<View style={[{ flexDirection: "row" }, this.props.containerStyle]}>
					{this.state.columns.map((col, index) => {
						return <Column key={index} index={index} ref={(ref) => {this.refArray[index]=ref}}
							keyExtractor={this.props.keyExtractor}
							renderItem={this.props.renderItem.bind(this)} />
					})}
				</View>
				{this.props.footer}
			</View>
		)
	}

	render() {
		return (
			<VirtualizedList
				{...this.props}
				bounces={false}
				onMomentumScrollEnd={this.props.onMomentumScrollEnd}
				listKey={"virtual_list_listKey_0"}
				data={[{ key: "virtual_list_0" }]}
				getItem={this.virtualizedListGetItem}
				getItemCount={this.virtualizedListGetItemCount}
				renderItem={this.virtualizedListRenderItem}
				contentContainerStyle={this.props.virtualizedListContentContainerStyle}
			/>
		)
	}
}
